/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * @record
 */
export function HeaderObj() { }
function HeaderObj_tsickle_Closure_declarations() {
    /** @type {?} */
    HeaderObj.prototype.label;
    /** @type {?} */
    HeaderObj.prototype.key;
}
export const /** @type {?} */ isJsons = (array) => Array.isArray(array) &&
    array.every(row => typeof row === 'object' && !(row instanceof Array));
export const /** @type {?} */ isArrays = (array) => Array.isArray(array) && array.every(row => Array.isArray(row));
/**
 * @param {?} array
 * @return {?}
 */
export function jsonsHeaders(array) {
    return Array.from(new Set(array.map(item => Object.keys(item)).reduce((a, b) => [...a, ...b], [])));
}
/**
 * @param {?} jsons
 * @param {?=} headers
 * @return {?}
 */
export function jsons2arrays(jsons, headers) {
    headers = headers || jsonsHeaders(jsons);
    // allow headers to have custom labels, defaulting to having the header data key be the label
    let /** @type {?} */ headerLabels = /** @type {?} */ (headers);
    let /** @type {?} */ headerKeys = /** @type {?} */ (headers);
    if (isJsons(headers)) {
        headerLabels = (/** @type {?} */ (headers)).map(header => header.label);
        headerKeys = (/** @type {?} */ (headers)).map(header => header.key);
    }
    const /** @type {?} */ data = jsons.map(object => headerKeys.map(header => (header in object ? object[header] : '')));
    return [headerLabels, ...data];
}
export const /** @type {?} */ elementOrEmpty = (element) => element || element === 0 ? element : '';
/**
 * @param {?} data
 * @param {?=} delimiter
 * @return {?}
 */
export function joiner(data, delimiter = ',') {
    return data
        .map((row, index) => row.map((element) => '"' + elementOrEmpty(element) + '"').join(delimiter))
        .join(`\n`);
}
/**
 * @param {?} data
 * @param {?=} headers
 * @param {?=} delimiter
 * @return {?}
 */
export function arrays2csv(data, headers, delimiter) {
    return joiner(headers ? [headers, ...data] : data, delimiter);
}
/**
 * @param {?} data
 * @param {?=} headers
 * @param {?=} delimiter
 * @return {?}
 */
export function jsons2csv(data, headers, delimiter) {
    return joiner(jsons2arrays(data, headers), delimiter);
}
/**
 * @param {?} data
 * @param {?=} headers
 * @param {?=} delimiter
 * @return {?}
 */
export function string2csv(data, headers, delimiter) {
    return headers ? `${headers.join(delimiter)}\n${data}` : data;
}
/**
 * @param {?} data
 * @param {?=} headers
 * @param {?=} delimiter
 * @return {?}
 */
export function toCSV(data, headers, delimiter) {
    if (isJsons(/** @type {?} */ (data))) {
        return jsons2csv(/** @type {?} */ (data), headers, delimiter);
    }
    if (isArrays(/** @type {?} */ (data))) {
        return arrays2csv(/** @type {?} */ (data), headers, delimiter);
    }
    if (typeof data === 'string') {
        return string2csv(data, /** @type {?} */ (headers), delimiter);
    }
    throw new TypeError(`Data should be a "String", "Array of arrays" OR "Array of objects" `);
}
/**
 * @param {?} data
 * @param {?=} uFEFF
 * @param {?=} headers
 * @param {?=} delimiter
 * @return {?}
 */
export function buildURI(data, uFEFF = true, headers, delimiter) {
    const /** @type {?} */ csv = toCSV(data, headers, delimiter);
    const /** @type {?} */ blob = new Blob([uFEFF ? '\uFEFF' : '', csv], { type: 'text/csv' });
    return URL.createObjectURL(blob);
}
//# sourceMappingURL=util.js.map
